"""Tuick, the Text User Interface for Compilers and checKers.

Tuick is a wrapper for compilers and checkers that integrates with fzf and your
text editor to provide fluid, keyboard-friendly, access to code error
locations.
"""

from importlib.metadata import version

__version__ = version("tuick")
